@@ -1,22 +1,17 @@
-"""This module contains helper classes and methods to assist with the
-collection of content to be posted to Mastodon."""
-# pylint: disable=E1136
+"""Helper classes and methods to assist with the collection of content to be posted to Mastodon."""
 import asyncio
 import configparser
 import hashlib
 import logging
 import os
 import re
-import sys
 from typing import Any
 from typing import Dict
 from typing import Iterable
 from typing import List
 from typing import Optional
 from typing import TypeVar
-from urllib.error import URLError
 from urllib.parse import urlsplit
-from urllib.request import urlopen
 
 import aiofiles
 import aiohttp
@@ -30,32 +25,28 @@ from imgurpython.helpers.error import ImgurClientError
 from tqdm import tqdm
 from tqdm.asyncio import tqdm as aiotqdm
 
-from . import __display_name__
 from . import PROGRESS_BAR_FORMAT
 from . import USER_AGENT
+from . import __display_name__
 from .control import Configuration
 from .control import Secret
 from .control import SubredditConfig
 
 logger = logging.getLogger(__display_name__)
 
-FATAL_TOOTBOT_ERROR = "Tootbot cannot continue, now shutting down"
-
 RH = TypeVar("RH", bound="RedditHelper")
 LMH = TypeVar("LMH", bound="LinkedMediaHelper")
 MA = TypeVar("MA", bound="MediaAttachment")
 
 
 async def get_file(img_url: str, file_path: str, progress_label: str) -> Optional[str]:
-    """Utility method to save a file located at img_url to a file located at
-    filepath.
+    """Save a file located at img_url to a file located at filepath.
 
-    Arguments:
-        img_url (string): url of imgur image to download
-        file_path (string): directory and filename where to save the downloaded image to
-        progress_label (string): Message to add to progress bar
+    :param img_url: url of imgur image to download
+    :param file_path: directory and filename where to save the downloaded image to
+    :param progress_label: Message to add to progress bar
 
-    Returns:
+    :returns:
         file_path (string): path to downloaded image or None if no image was downloaded
     """
     logger.debug(
@@ -100,9 +91,10 @@ async def get_file(img_url: str, file_path: str, progress_label: str) -> Optiona
 
 
 async def get_secrets(config_dir: str) -> Dict[str, Secret]:
-    """Helper method to collect all api secrets either from already store
-    secrets files or from user input."""
+    """Collect all api secrets either from already store secrets files or from user input.
 
+    :param config_dir: directory to check read config.ini file from
+    """
     secrets = {}
     secrets["reddit"] = await _get_reddit_secret(config_dir=config_dir)
     secrets["imgur"] = _get_imgur_secret(config_dir=config_dir)
@@ -111,8 +103,10 @@ async def get_secrets(config_dir: str) -> Dict[str, Secret]:
 
 
 async def _get_reddit_secret(config_dir: str) -> Secret:
-    """Helper method to collect reddit secret from stored secrets file or from
-    user input."""
+    """Collect reddit secret from stored secrets file or from user input.
+
+    :param config_dir: directory to check read config.ini file from
+    """
     reddit_secrets_file = f"{config_dir}/reddit.secret"  # pragma: allowlist secret
     reddit_config = configparser.ConfigParser()
     if not os.path.exists(reddit_secrets_file):
@@ -124,29 +118,24 @@ async def _get_reddit_secret(config_dir: str) -> Secret:
             input("[ .. ] Enter Reddit client secret: ").split()
         )
         # Make sure authentication is working
-        try:
-            # create Reddit api connection and load posts from announcements subreddit
-            # to confirm reddit connection works
-            reddit_client = asyncpraw.Reddit(
-                user_agent="Tootbot",
-                client_id=reddit_agent,
-                client_secret=reddit_client_secret,
-            )
-            subreddit = await reddit_client.subreddit("announcements")
-            async for _post in subreddit.hot():
-                continue
+        # create Reddit api connection and load posts from announcements subreddit
+        # to confirm reddit connection works
+        reddit_client = asyncpraw.Reddit(
+            user_agent="Tootbot",
+            client_id=reddit_agent,
+            client_secret=reddit_client_secret,
+        )
+        subreddit = await reddit_client.subreddit("announcements")
+        async for _post in subreddit.hot():
+            continue
 
-            # It worked, so save the keys to a file
-            reddit_config["Reddit"] = {
-                "Agent": reddit_agent,
-                "ClientSecret": reddit_client_secret,
-            }
-            with open(reddit_secrets_file, "w", encoding="utf8") as new_reddit_secrets:
-                reddit_config.write(new_reddit_secrets)
-        except asyncprawcore.AsyncPrawcoreException as reddit_exception:
-            logger.error("Error while logging into Reddit: %s", reddit_exception)
-            logger.error(FATAL_TOOTBOT_ERROR)
-            sys.exit(1)
+        # It worked, so save the keys to a file
+        reddit_config["Reddit"] = {
+            "Agent": reddit_agent,
+            "ClientSecret": reddit_client_secret,
+        }
+        with open(reddit_secrets_file, "w", encoding="utf8") as new_reddit_secrets:
+            reddit_config.write(new_reddit_secrets)
     else:
         # Read API keys from secret file
         reddit_config.read(reddit_secrets_file)
@@ -158,16 +147,18 @@ async def _get_reddit_secret(config_dir: str) -> Secret:
 
 
 def _get_imgur_secret(config_dir: str) -> Secret:
-    """_get_imgur_secrets checks if the Imgur api secrets file exists.
+    """Check if the Imgur api secrets file exists.
 
     - If the file exists, this method reads the imgur secrets file and returns the
       secrets a Secret dataclass.
     - If the file doesn't exist it asks the user over stdin to supply these values
       and then saves them into the imgur_secrets file
 
-    Returns:
+    :param config_dir: directory to check read config.ini file from
+
+    :returns:
         api_secrets (Secret): instance of Secret class containing the api secrets
-        to work with imgut
+        to work with imgur
     """
     secrets_file = f"{config_dir}/imgur.secret"  # pragma: allowlist secret
     if not os.path.exists(secrets_file):
@@ -179,23 +170,18 @@ def _get_imgur_secret(config_dir: str) -> Secret:
             input("[ .. ] Enter Imgur client secret: ").split()
         )
         # Make sure authentication is working
-        try:
-            imgur_client = ImgurClient(imgur_client_id, imgur_client_secret)
+        imgur_client = ImgurClient(imgur_client_id, imgur_client_secret)
 
-            # If this call doesn't work, it'll throw an ImgurClientError
-            imgur_client.get_album("dqOyj")
-            # It worked, so save the keys to a file
-            imgur_config = configparser.ConfigParser()
-            imgur_config["Imgur"] = {
-                "ClientID": imgur_client_id,
-                "ClientSecret": imgur_client_secret,
-            }
-            with open(secrets_file, "w", encoding="UTF-8") as file:
-                imgur_config.write(file)
-        except ImgurClientError as imgur_error:
-            logger.error("Error while logging into Imgur: %s", imgur_error)
-            logger.error(FATAL_TOOTBOT_ERROR)
-            sys.exit(1)
+        # If this call doesn't work, it'll throw an ImgurClientError
+        imgur_client.get_album("dqOyj")
+        # It worked, so save the keys to a file
+        imgur_config = configparser.ConfigParser()
+        imgur_config["Imgur"] = {
+            "ClientID": imgur_client_id,
+            "ClientSecret": imgur_client_secret,
+        }
+        with open(secrets_file, "w", encoding="UTF-8") as file:
+            imgur_config.write(file)
     else:
         # Read API keys from secret file
         imgur_config = configparser.ConfigParser()
@@ -209,17 +195,23 @@ def _get_imgur_secret(config_dir: str) -> Secret:
 
 class RedditHelper:
     """RedditHelper provides methods to collect data / content from reddit to
-    then post on Mastodon."""
+    then post on Mastodon.
+    """
 
     # Check if reddit access details in 'reddit.secret' file has already been set up
     # and load it, otherwise guide user through setting it up.
     def __init__(self: RH, config: Configuration, api_secret: Secret) -> None:
+        """Initialise RedditHelper instance.
+
+        :param config: Configuration settings
+        :param api_secret: Secrets to use with reddit api
+        """
         self.config = config
         self.posts: Dict[str, asyncpraw.models.Submission] = {}
         self.api_secret = api_secret
 
     async def get_all_reddit_posts(self: RH) -> None:
-        """Collects posts from all configured subreddits."""
+        """Collect posts from all configured subreddits."""
         tasks = []
 
         reddit_con = asyncpraw.Reddit(
@@ -249,7 +241,14 @@ class RedditHelper:
         subreddit: SubredditConfig,
         reddit_con: asyncpraw.Reddit,
     ) -> Dict[str, asyncpraw.models.Submission]:
-        """Collects posts considered hot from configured sub/multi-reddits."""
+        """Collect posts considered hot from configured sub/multi-reddits.
+
+        :param subreddit: subreddits to check for posts to x-post
+        :param reddit_con: API reference
+
+        :returns:
+        Dictionary with str of tags to use and subreddit posts
+        """
         posts = {}
         logger.debug("Retrieving posts from %s", subreddit.name)
         try:
@@ -269,8 +268,7 @@ class RedditHelper:
         return posts
 
     async def winnow_reddit_posts(self: RH) -> None:
-        """Filters out reddit posts according to configuration and whether it
-        has already been tooted."""
+        """Filter out reddit posts according to configuration and whether it has already been posted."""
         recorder = self.config.bot.post_recorder
         nsfw_allowed = self.config.reddit.nsfw_allowed
         self_posts_allowed = self.config.reddit.self_posts
@@ -287,7 +285,6 @@ class RedditHelper:
         ):
             post_ids = list(posts.keys())
             for post_id in post_ids:
-
                 if posts[post_id].over_18 and not nsfw_allowed:
                     # Skip over NSFW posts if they are disabled in the config file
                     logger.debug("Skipping %s, it is marked as NSFW", post_id)
@@ -333,14 +330,16 @@ class RedditHelper:
         determined from the text of the reddit submission, if a promo message
         should be included, and any hashtags.
 
-        Arguments:
-            submission (Submission): PRAW Submission object for the reddit post we are
+        :param submission: PRAW Submission object for the reddit post we are
             determining the mastodon toot text for.
-            max_len: (int): The maximum length the text for the mastodon toot can be.
-            add_hash_tags (str): additional hashtags to be added to global hashtags
+        :param max_len: The maximum length the text for the mastodon toot can be.
+        :param add_hash_tags: additional hashtags to be added to global hashtags
             defined in config file. The hashtags must be comma delimited
-            promo_message (str): Any promo message that must be added to end of caption.
+        :param promo_message: Any promo message that must be added to end of caption.
             Set to None if no promo message to be added
+
+        :returns:
+        Caption to use in fediverse post
         """
         logger.debug(
             "RedditHelper.get_caption("
@@ -382,46 +381,48 @@ class RedditHelper:
         if self.config.mastodon_config.use_tags:
             caption += hashtag_string
         if self.config.mastodon_config.use_backlink:
-            caption += submission.shortlink
+            if self.config.mastodon_config.link_to_media:
+                caption += submission.url
+            else:
+                caption += submission.shortlink
         caption += promo_string
         return caption
 
 
 class LinkedMediaHelper:
     """ImgurHelper provides methods to collect data / content from Imgur and
-    Gfycat."""
+    Gfycat.
+    """
 
     def __init__(
         self: LMH,
         config: Configuration,
         imgur_secrets: Secret,
     ) -> None:
+        """Initialise LinkedMediaHelper instance.
+
+        :param config: Configuration settings
+        :param imgur_secrets: secrets to use with imgur api
+
+        """
         self.save_dir = config.media.folder
 
-        try:
-            self.imgur_client = ImgurClient(
-                client_id=imgur_secrets.client_id,
-                client_secret=imgur_secrets.client_secret,
-            )
-        except ImgurClientError as imgur_error:
-            logger.error("Error on creating ImgurClient: %s", imgur_error)
-            logger.error(FATAL_TOOTBOT_ERROR)
-            sys.exit(1)
+        self.imgur_client = ImgurClient(
+            client_id=imgur_secrets.client_id,
+            client_secret=imgur_secrets.client_secret,
+        )
 
     async def get_imgur_image(
         self: LMH, img_url: str, max_images: int = 4
     ) -> List[str]:
-        """get_imgur_image downloads images from imgur.
+        """Download images from imgur.
 
-        Arguments:
-            img_url: url of imgur image to download
-            max_images: maximum number of images to download and process, Defaults to 4
+        :param img_url: url of imgur image to download
+        :param max_images: maximum number of images to download and process, Defaults to 4
 
-        Returns:
-            file_paths (string): path to downloaded image or None if no image was
-            downloaded
+        :returns:
+        path to downloaded image or None if no image was downloaded
         """
-
         # Working demo of regex: https://regex101.com/r/G29uGl/2
         regex = r"(?:.*)imgur\.com(?:\/gallery\/|\/a\/|\/)(.*?)(?:\/.*|\.|$)"
         regex_match = re.search(regex, img_url, flags=0)
@@ -478,15 +479,13 @@ class LinkedMediaHelper:
         return imgur_paths
 
     def _get_image_urls(self: LMH, img_url: str, imgur_id: str) -> List[str]:
-        """_get_image_urls builds a list of urls of all Imgur images identified
-        by imgur_id.
+        """Build a list of urls of all Imgur images identified by imgur_id.
 
-        Arguments:
-            img_url: URL to IMGUR post
-            imgur_id: ID for IMGUR post
+        :param img_url: URL to IMGUR post
+        :param imgur_id: ID for IMGUR post
 
-        Returns:
-            imgur_urls: List of urls to images of Imgur post identified byr imgur_id
+        :returns:
+        imgur_urls: List of urls to images of Imgur post identified byr imgur_id
         """
         image_urls = []
         try:
@@ -504,14 +503,12 @@ class LinkedMediaHelper:
         return image_urls
 
     def _check_imgur_gif(self: LMH, file_path: str) -> bool:
-        """_check_imgur_gif checks if a file downloaded from imgur is indeed a
-        gif. If file is not a gif, remove the file.
+        """Check if a file downloaded from imgur is indeed a gif. If file is not a gif, remove the file.
 
-        Arguments:
-            file_path: file name and path to downloaded image
+        :param file_path: file name and path to downloaded image
 
-        Returns:
-             True if downloaded image is indeed a GIF, otherwise returns False
+        :returns:
+        True if downloaded image is indeed a GIF, otherwise returns False
         """
         mime = magic.from_file(filename=file_path, mime=True)
         if mime != "image/gif":
@@ -525,14 +522,12 @@ class LinkedMediaHelper:
         return True
 
     def get_gfycat_image(self: LMH, img_url: str) -> Optional[str]:
-        """get_gfycat_image downloads full resolution images from gfycat.
+        """Download full resolution images from gfycat.
 
-        Arguments:
-            img_url (string): url of gfycat image to download
+        :param img_url: url of gfycat image to download
 
-        Returns:
-            file_path (string): path to downloaded image or None if no image was
-            downloaded
+        :returns:
+        file_path (string): path to downloaded image or None if no image was downloaded
         """
         logger.debug("LinkedMediaHelper.get_gfycat_image(img_url=%s)", img_url)
 
@@ -543,15 +538,12 @@ class LinkedMediaHelper:
         return saved_paths
 
     async def get_reddit_image(self: LMH, img_url: str) -> Optional[str]:
-        """get_reddit_image downloads full resolution images from i.reddit or
-        reddituploads.com.
+        """Download full resolution images from i.reddit or reddituploads.com.
 
-        Arguments:
-            img_url (string): url of imgur image to download
+        :param img_url (string): url of imgur image to download
 
-        Returns:
-            file_path (string): path to downloaded image or None if no image was
-            downloaded
+        :returns:
+        file_path (string): path to downloaded image or None if no image was downloaded
         """
         file_name = os.path.basename(urlsplit(img_url).path)
         file_extension = os.path.splitext(img_url)[1].lower()
@@ -583,53 +575,51 @@ class LinkedMediaHelper:
         reddit_post: Submission,
         max_images: int = 4,
     ) -> List[Optional[str]]:
-        """get_reddit_gallery downloads up to max_images images from a reddit
-        gallery post and returns a List of file_paths downloaded images.
+        """Download up to max_images images from a reddit gallery post and returns a List of file_paths
+        downloaded images.
 
-        Arguments:
-            reddit_post (reddit_post):  reddit post / submission object
-            max_images (int): [optional] maximum number of images to download.
-            Default is 4
+        :param reddit_post:  reddit post / submission object
+        :param max_images: [optional] maximum number of images to download. Default is 4
 
-        Returns:
-            file_paths (List[str]) a list of the paths to downloaded files. If no
-            images have been downloaded, and empty list will be returned.
+        :returns:
+        file_paths (List[str]) a list of the paths to downloaded files. If no images have been downloaded, and empty
+        list will be returned.
         """
         file_paths: List[Optional[str]] = []
-        gallery_items: List[Dict[str, Any]] = reddit_post.gallery_data["items"]
-        tasks: List[Any] = []
-        for item in gallery_items:
-            media_id = item["media_id"]
-            meta = reddit_post.media_metadata[media_id]
-            logger.debug("Media Metadata: %s", meta)
-            if "e" in meta and meta["e"] == "Image":
-                source = meta["s"]
-                save_path = (
-                    self.save_dir + "/" + media_id + "." + meta["m"].split("/")[1]
-                )
-                logger.debug(
-                    "Gallery file_path, source: %s - %s", save_path, source["u"]
-                )
-                tasks.append(get_file(source["u"], save_path, f"image {len(tasks)+1}"))
+        if reddit_post.gallery_data.get("items"):
+            gallery_items: List[Dict[str, Any]] = reddit_post.gallery_data["items"]
+            tasks: List[Any] = []
+            for item in gallery_items:
+                media_id = item["media_id"]
+                meta = reddit_post.media_metadata[media_id]
+                logger.debug("Media Metadata: %s", meta)
+                if "e" in meta and meta["e"] == "Image":
+                    source = meta["s"]
+                    save_path = (
+                        self.save_dir + "/" + media_id + "." + meta["m"].split("/")[1]
+                    )
+                    logger.debug(
+                        "Gallery file_path, source: %s - %s", save_path, source["u"]
+                    )
+                    tasks.append(
+                        get_file(source["u"], save_path, f"image {len(tasks)+1}")
+                    )
 
-                if len(tasks) == max_images:
-                    break
+                    if len(tasks) == max_images:
+                        break
 
-        print("Donwloading images from reddit gallery")
-        file_paths = await asyncio.gather(*tasks)
+            print("Downloading images from reddit gallery")
+            file_paths = await asyncio.gather(*tasks)
 
         return file_paths
 
     def get_reddit_video(self: LMH, reddit_post: Submission) -> Optional[str]:
-        """get_reddit_video downloads full resolution video from i.reddit or
-        reddituploads.
+        """Download full resolution video from i.reddit or reddituploads.
 
-        Arguments:
-            reddit_post (reddit_post): reddit post / submission object
+        :param reddit_post: reddit post / submission object
 
-        Returns:
-            file_path (string): path to downloaded video or None if no image was
-            downloaded
+        :returns:
+        file_path (string): path to downloaded video or None if no image was downloaded
         """
         logger.debug(
             "LinkedMediaHelper.get_reddit_video(reddit_post = %s)",
@@ -652,14 +642,13 @@ class LinkedMediaHelper:
 
     @staticmethod
     def _get_video_with_yt_dlp(video_url: str, save_as: str) -> Optional[str]:
-        """Downloads video files with embedded yt-dlp.
+        """Download video files with embedded yt-dlp.
 
-        Arguments:
-            video_url (string): URL for video to download
-            save_as (string): path and file name excluding file extension for yt-dlp to
+        :param video_url: URL for video to download
+        :param save_as: path and file name excluding file extension for yt-dlp to
                 download video file to
 
-        Returns:
+        :returns:
             (string) containing file name inclusive file extension and path where yt-dlp
                 has saved the downloaded video.
                 This can be None if yt-dlp has been unsuccessful.
@@ -707,14 +696,12 @@ class LinkedMediaHelper:
         return yt_dlp_filepath
 
     async def get_giphy_image(self: LMH, img_url: str) -> Optional[str]:
-        """get_giphy_image downloads full or low resolution image from giphy.
+        """Download full or low resolution image from giphy.
 
-        Arguments:
-            img_url (string): url of giphy image to download
+        :param img_url: url of giphy image to download
 
-        Returns:
-            file_path (string): path to downloaded image or None if no image was
-            downloaded
+        :returns:
+        file_path (string): path to downloaded image or None if no image was downloaded
         """
         # Working demo of regex: https://regex101.com/r/o8m1kA/2
         regex = (
@@ -743,16 +730,15 @@ class LinkedMediaHelper:
         return saved_paths
 
     async def get_generic_image(self: LMH, img_url: str) -> Optional[str]:
-        """get_generic_image downloads image or video from a generic url to a
-        media file.
+        """Download image or video from a generic url to a media file.
 
-        Arguments:
-            img_url (string): url to image or video file
+        :param img_url: url to image or video file
 
-        Returns:
-            file_path (string): path to downloaded video or None if no image was
-            downloaded
+        :returns:
+        file_path (string): path to downloaded video or None if no image was downloaded
         """
+        logger.debug("LinkedMediaHelper.get_generic_image(img_url=%s)", img_url)
+
         # First check if URL starts with http:// or https://
         regex = r"^https?://"
         match = re.search(regex, img_url, flags=0)
@@ -768,17 +754,21 @@ class LinkedMediaHelper:
             "image/webp",
             "video/mp4",
         )
-        try:
-            # img_url has already been checked that it starts with "http://" or
-            # "https://" above.
-            with urlopen(img_url) as img_site:  # nosec B310
-                meta = img_site.info()
 
-        except (URLError, UnicodeEncodeError) as url_error:
-            logger.error("Error while opening URL %s", url_error)
+        try:
+            async with aiohttp.ClientSession(
+                raise_for_status=True,
+                read_timeout=30,
+            ) as client:
+                response = await client.head(url=img_url)
+                headers = response.headers
+                content_type = headers.get("content-type", None)
+
+        except (aiohttp.ClientError, asyncio.exceptions.TimeoutError) as error:
+            logger.error("Error while opening URL: %s ", error)
             return None
 
-        if meta["content-type"] not in image_formats:
+        if content_type not in image_formats:
             logger.debug("URL does not point to a valid image file: %s", img_url)
             return None
 
@@ -798,19 +788,24 @@ class LinkedMediaHelper:
 
 class MediaAttachment:
     """MediaAttachment contains code to retrieve the appropriate images or
-    videos to include in a reddit post to be shared on Mastodon."""
+    videos to include in a reddit post to be shared on Mastodon.
+    """
 
     def __init__(
         self: MA, reddit_post: Submission, image_helper: LinkedMediaHelper
     ) -> None:
+        """Initialise MediaAttachment instance.
 
+        :param reddit_post: reddit post / submission
+        :param image_helper: MediaHelper to deal with images or videos linked in reddit post
+        """
         self.media_paths: Dict[str, str] = {}
         self.reddit_post = reddit_post
         self.media_url = self.reddit_post.url
         self.image_helper = image_helper
 
     async def get_media_files(self: MA) -> None:
-        """Downloads media files linked to given reddit_post."""
+        """Download media files linked to given reddit_post."""
         logger.debug("URL for post(%s): %s ", self.reddit_post, self.media_url)
 
         media_paths = await self.get_media()
@@ -829,8 +824,7 @@ class MediaAttachment:
                 self.media_paths[sha256.hexdigest()] = media_path
 
     def destroy(self: MA) -> None:
-        """Removes any files downloaded and clears out the object
-        attributes."""
+        """Remove any files downloaded and clear out the object attributes."""
         try:
             for media_path in self.media_paths.values():
                 if media_path is not None:
@@ -843,10 +837,9 @@ class MediaAttachment:
         self.media_url = None
 
     def destroy_one_attachment(self: MA, checksum: str) -> None:
-        """Removes file with checksum downloaded.
+        """Remove file with checksum downloaded.
 
-        Arguments:
-            checksum (string): key to media_paths dictionary for file to be removed.
+        :param checksum: key to media_paths dictionary for file to be removed.
         """
         try:
             media_path = self.media_paths[checksum]
@@ -858,8 +851,11 @@ class MediaAttachment:
             logger.error("Error while deleting media file: %s", delete_error)
 
     async def get_media(self: MA) -> List[Optional[str]]:
-        """Determines which method to call depending on which site the
-        media_url is pointing to."""
+        """Determine which method to call depending on which site the media_url is pointing to.
+
+        :returns:
+        list of file-paths of downloaded images. Can be None
+        """
         if not os.path.exists(self.image_helper.save_dir):
             os.makedirs(self.image_helper.save_dir)
             logger.debug(
