@@ -1,5 +1,6 @@
 """Classes / Methods to assist with monitoring continued operation of
-tootboot."""
+tootboot.
+"""
 import logging
 from typing import Optional
 from typing import TypeVar
@@ -16,20 +17,23 @@ logger = logging.getLogger(__display_name__)
 
 class HealthChecks:
     """Class to make monitoring the operation of tootboot with Healthchecks
-    (healthchecks.io) easier."""
+    (healthchecks.io) easier.
+    """
 
     def __init__(self: HC, config: Configuration) -> None:
+        """Initialise Healthcheck instance.
+
+        :param config: Configuration settings.
+        """
         self.base_url = config.health.base_url
         self.uid = config.health.uuid
 
     async def check(self: HC, data: str = "", check_type: Optional[str] = None) -> None:
         """Check in with a Healthchecks installation.
 
-        Keyword Arguments:
-            data (string):
-                Data to send along with the check in. Can be used to include a short
+        :param data: Data to send along with the check in. Can be used to include a short
                 status along with the check in.
-            check_type (string):
+        :param check_type:
                 - Type of check in. An empty (None) check_type signals an ok check in
                   and also the successful completion of an earlier 'start' check in
                   type.
@@ -50,13 +54,13 @@ class HealthChecks:
             logger.error('During Monitoring "OK Ping" we got: %s', check_error)
 
     async def check_ok(self: HC, data: str = "") -> None:
-        """Convenience method to signal an OK completion of a process."""
+        """Signal an OK completion of a process."""
         await self.check(data=data)
 
     async def check_start(self: HC, data: str = "") -> None:
-        """Convenience method to signal the start of a process."""
+        """Signal the start of a process."""
         await self.check(data=data, check_type="start")
 
     async def check_fail(self: HC, data: str = "") -> None:
-        """Convenience method to signal the failure of a process."""
+        """Signal the failure of a process."""
         await self.check(data=data, check_type="fail")
